#include "metadatadialog.h"
#include "ui_metadatadialog.h"
#include <pro.h>
#include <QPushButton>

MetadataDialog::MetadataDialog(DmxWidget *device, QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::MetadataDialog)
    , dev(device)
{
    ui->setupUi(this);

    ui->breakSpinBox->setSingleStep(ENTTEC::Pro::DMX_BREAK_INTERVAL);
    ui->breakSpinBox->setMinimum(ENTTEC::Pro::DMX_BREAK_INTERVAL * ENTTEC::Pro::DMX_BREAK_MIN);
    ui->breakSpinBox->setMaximum(ENTTEC::Pro::DMX_BREAK_INTERVAL * ENTTEC::Pro::DMX_BREAK_MAX);
    ui->breakSpinBox->setValue(device->txBreakTime());

    ui->mabSpinBox->setSingleStep(ENTTEC::Pro::DMX_MAB_INTERVAL);
    ui->mabSpinBox->setMinimum(ENTTEC::Pro::DMX_MAB_INTERVAL * ENTTEC::Pro::DMX_MAB_MIN);
    ui->mabSpinBox->setMaximum(ENTTEC::Pro::DMX_MAB_INTERVAL * ENTTEC::Pro::DMX_MAB_MAX);
    ui->mabSpinBox->setValue(device->txMabTime());

    ui->rateSpinBox->setSingleStep(1);
    ui->rateSpinBox->setMinimum(0);         // zero rate for fastest tx possible
    ui->rateSpinBox->setMaximum(ENTTEC::Pro::DMX_RATE_MAX);
    ui->rateSpinBox->setValue(device->txRate());

    connect(ui->buttonBox->button(QDialogButtonBox::RestoreDefaults), &QPushButton::clicked,
                                  this, [this](){
        ui->breakSpinBox->setValue(17 * ENTTEC::Pro::DMX_BREAK_INTERVAL);   // match OEM
        ui->mabSpinBox->setValue(10 * ENTTEC::Pro::DMX_MAB_INTERVAL);       // match OEM
        ui->rateSpinBox->setValue(40);                                      // match OEM
    });

    connect(this, &QDialog::accepted, this, [this](){
        dev->setTxBreakTime(ui->breakSpinBox->value());
        dev->setTxMabTime(ui->mabSpinBox->value());
        dev->setTxRate(ui->rateSpinBox->value());
        dev->setParameters();
    });

}

MetadataDialog::~MetadataDialog()
{
    delete ui;
}
