#pragma once

#include <QAbstractTableModel>
#include <universe.h>

class UniverseModel
    : public QAbstractTableModel
{
    Q_OBJECT
public:
    explicit UniverseModel(QObject *parent = nullptr,
                           std::shared_ptr<DMX::Universe> universe = nullptr);
    // Model overrides:
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role = Qt::DisplayRole) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override;
    QVariant data(const QModelIndex &index,
                  int role = Qt::DisplayRole) const override;
    bool setData(const QModelIndex &index, const QVariant &value,
                 int role = Qt::EditRole) override;
    Qt::ItemFlags flags(const QModelIndex& index) const override;

    /// @brief The data_modes enum
    enum data_modes {
        Decimal,
        Hex,
        Percent
    };
    Q_ENUM(data_modes)
    data_modes dataMode();

public slots:
    void setDataMode(const UniverseModel::data_modes mode);

private:
    std::shared_ptr<DMX::Universe> universe_;
    data_modes data_mode_;

    std::shared_ptr<void> data_change_token;
};
