/*
  transmitter.h

  Copyright (c) 2023 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include <cstdint>

namespace ANALOG {

/**
 * @brief A single-channel 0-10V controller.
 */
struct Transmitter
{
  /**
   * \cite ANALOG 6.1.1 Amplitude (Transmitter)
   * Zero volts shall represent the full off condition and ten volts shall represent the
   * full on condition.
   */
  static const int16_t zero     =     0; //!< mV,  0.0 V
  static const int16_t zero_max =   200; //!< mv,  0.2 V
  static const int16_t full     = 10000; //!< mV, 10.0 V

  /**
   * @brief Convert an 8-bit level to millivolts.
   * @param level
   * @return
   */
  int16_t millivolts(uint8_t level) const {
    if (level == 0)
      return zero;
    if (level == UINT8_MAX)
      return full;
    return ((full - zero_max) * (level / UINT8_MAX)) + zero_max;
  };
};

} // namespace ANALOG
