/*
  property.h

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "message.h"
#include "parameterdescription.h"

#include <functional>

namespace RDM {

using PidAction = std::function<void(MsgPair)>;     //!< Function that performs action on a Message.

/**
 * @brief The Parameter class
 */
class Parameter
{
public:
  explicit Parameter(const ParameterDescription metadata);

  const ParameterDescription info;          //!< Parameter ID
  bool isManufacturer() const;

  void dispatch(MsgPair msg) const;

  // responder behavior
  void onDisc(const PidAction action);
  void onGet(const PidAction action);
  void onSet(const PidAction action);

  // controller behavior
  void onDiscResponse(const PidAction action);
  void onGetResponse(const PidAction action);
  void onSetResponse(const PidAction action);

private:
  // responder actions
  PidAction action_discover_;
  PidAction action_get_;
  PidAction action_set_;

  // controller actions
  PidAction action_discover_response_;
  PidAction action_get_response_;
  PidAction action_set_response_;
};

} // namespace RDM
