/*
  osc/bundle.h

  Copyright (c) 2022 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include <bufferstream.h>
#include <memory>
#include "message.h"
#include <vector>

namespace OSC {

/**
 * @brief The Bundle struct
 *
 * \cite Spec10 An OSC Bundle consists of the OSC-string “#bundle” followed by an OSC Time Tag,
 * followed by zero or more OSC Bundle Elements.
 *
 * Inherits from Message as a Bundle may also be a Bundle Element.
 */
struct Bundle
  : Message
{
  ///@brief Bundle
  Bundle() { address_pattern = "#bundle"; }

  virtual size_t streamSize() const override;
  virtual void iStream(std::shared_ptr<bufferstream>) override;
  virtual void oStream(std::shared_ptr<bufferstream>) const override;

  timetag time_tag;                               //!< delay excution of elements until this time
  std::vector<std::shared_ptr<Message>> elements; //!< bundled elements, either messages or bundles
};

} // namespace OSC
