/*
  osc/message.h

  Copyright (c) 2022 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include <argument.h>
#include <bufferstream.h>
#include <vector>

namespace OSC {

/**
 * @brief The Message struct
 *
 * \cite Spec10 An OSC message consists of an OSC Address Pattern followed by an OSC
 * Type Tag String followed by zero or more OSC Arguments.
 */
struct Message
  : streamable
{
    std::string address_pattern;            //!<An OSC-string beginning with the character ‘/’.
    std::string type_tag() const;
    std::vector<std::shared_ptr<Argument>> arguments; //!< zero or more Arguments.

    static std::vector<std::shared_ptr<Argument>> createArguments(std::string);
    static void readString(std::shared_ptr<bufferstream> buffer, std::string &string);
    static void writeString(std::shared_ptr<bufferstream> buffer, const std::string &string);

    void addArgument(std::shared_ptr<Argument> arg);
    void addArgument(const int32_t val);                // i
    void addArgument(const float val);                  // f
    void addArgument(const std::string str);            // s
    void addArgument(const uint8_t *blob, size_t size); // b
    void addArgument(const int64_t val);                // h
    void addArgument(const double val);                 // d
    void addArgument(const char val);                   // c
    void addArgument(const bool state);                 // T, F

    virtual size_t streamSize() const override;
    virtual void iStream(std::shared_ptr<bufferstream>) override;
    virtual void oStream(std::shared_ptr<bufferstream>) const override;
};

} // namespace OSC
