/*
  osc/method.h

  Copyright (c) 2023 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "message.h"
#include <functional>
#include <list>
#include <string>

namespace OSC {

/**
 * @brief The Method class
 *
 * \cite Spec10 OSC Methods are the potential destinations of OSC messages received by the OSC
 * server and correspond to each of the points of control that the application makes available.
 */
class Method
{
public:
    explicit Method(Method *parent = nullptr, std::string name = "");
    virtual ~Method();

    void addChild(Method *child);

    std::string address() const;
    std::string name() const;
    bool setName(std::string name);

    bool dispatch(std::list<std::string> patterns, const std::shared_ptr<Message> msg) const;

    std::shared_ptr<void> onTrigger(const std::function<void(std::shared_ptr<Message>)>);

protected:
    void trigger(std::shared_ptr<Message>) const;
    bool matchName(const std::string &pattern) const;
    bool isRoot() const;
    bool isContainer() const;

private:
    Method *parent_;
    std::vector<Method*> children_;
    std::string name_;

    std::vector<std::weak_ptr<const std::function<void(std::shared_ptr<Message>)>>> cb_trigger;
};

} // namespace OSC
