/*
  osc/server.h

  Copyright (c) 2023 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include <bufferstream.h>
#include "bundle.h"
#include "message.h"
#include "method.h"

namespace OSC {

/**
 * @brief \cite Spec10 Any application that receives OSC Packets is an OSC Server.
 *
 * As of 2021, Adrian Freed and Matt Wright recommend the terminology “OSC Receiver” for
 * anything that can receive OSC packets.
 */
class Receiver
{
public:
    explicit Receiver();
    virtual ~Receiver();

    void dispatch(const std::shared_ptr<Message> msg) const;

    /**
     * @brief rootMethod
     * @return
     */
    Method* rootMethod() { return address_space_; }

    void rxPacket(std::shared_ptr<bufferstream> buffer) const;
    void rxPacketStream(std::shared_ptr<bufferstream> buffer) const;
    void rxPacketSLIP(std::shared_ptr<bufferstream> buffer) const;

protected:
    void scheduleBundle(const std::shared_ptr<Bundle> bundle) const;

private:
    Method* address_space_;
};

} // namespace OSC
