/*
  osc/client.h

  Copyright (c) 2022 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include <bufferstream.h>
#include <functional>
#include <memory>
#include "message.h"

namespace OSC {

/**
 * @brief \cite Spec10 Any application that sends OSC Packets is an OSC Client.
 *
 * As of 2021, Adrian Freed and Matt Wright recommend the terminology “OSC Sender” for
 * anything that sends OSC packets.
 */
class Sender
{
public:
    explicit Sender();
    virtual ~Sender();

    void send(std::shared_ptr<Message>) const;
    void send(std::vector<std::shared_ptr<Message>>,
              std::chrono::time_point<std::chrono::system_clock>
                                                        = std::chrono::system_clock::now()) const;
    std::shared_ptr<void> setSender(const std::function<void(std::shared_ptr<bufferstream>)>);


private:
    std::weak_ptr<const std::function<void(std::shared_ptr<bufferstream>)>> sender_;
};

} // namespace OSC
